'use strict';

Object.defineProperty(exports, "__esModule", {
				value: true
});

var _eos = require('./eos');

var eos = _interopRequireWildcard(_eos);

var _Identity = require('../models/Identity');

var _Identity2 = _interopRequireDefault(_Identity);

var _Reputable = require('../models/Reputable');

var _helpers = require('../util/helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const getIdentity = (() => {
				var _ref = _asyncToGenerator(function* (username) {
								return yield eos.read({
												table: 'ids',
												key_type: 'i64',
												index_position: 2,
												index: (0, _helpers.fingerprinted)(username),
												limit: 1,
												model: _Identity2.default,
												firstOnly: true
								});
				});

				return function getIdentity(_x) {
								return _ref.apply(this, arguments);
				};
})();

class IdentityService {

				constructor() {
								this.paymentSymbol = 'EOS';
				}

				setSymbol(symbol) {
								this.paymentSymbol = symbol;
				}
				validName(name) {
								return (/^[a-zA-Z0-9_-]{3,20}$/.test(name)
								);
				}

				/***
     * Gets an identity based on a username
     * @param username
     * @returns {Promise<*>}
     */
				get(username) {
								return _asyncToGenerator(function* () {
												return getIdentity(username);
								})();
				}

				getById(id) {
								return _asyncToGenerator(function* () {
												return yield eos.read({
																table: 'ids',
																index: id,
																limit: 1,
																model: _Identity2.default,
																firstOnly: true
												});
								})();
				}

				getByAccount(accountName) {
								return _asyncToGenerator(function* () {
												return yield eos.read({
																table: 'ids',
																key_type: 'i64',
																index_position: 3,
																index: accountName,
																limit: 1,
																model: _Identity2.default,
																firstOnly: true
												});
								})();
				}

				/***
     * Pays for and identifies an identity
     * @param username
     * @param key
     * @returns {Promise<*>}
     */
				payAndIdentify(username, key) {
								var _this = this;

								return _asyncToGenerator(function* () {
												return eos.writer.transaction(['eosio.token', 'ridlridlridl'], function (contracts) {
																contracts.eosio_token.transfer(eos.account.name, 'ridlridlridl', `1.0000 ${_this.paymentSymbol}`, '', eos.options);
																contracts.ridlridlridl.identify(eos.account.name, username, key, eos.options);
												});
								})();
				}

				/***
     * Changes the identity key
     * @param username
     * @param key
     * @returns {Promise<T | boolean>}
     */
				changekey(username, key) {
								var _this2 = this;

								return _asyncToGenerator(function* () {
												return eos.contract.changekey(username, key, eos.options).then(function () {
																return _this2.get(username);
												}).catch(function (err) {
																return false;
												});
								})();
				}

				/***
     * Changes the identity account
     * @param username
     * @param key
     * @returns {Promise<T | boolean>}
     */
				changeacc(username, newAccountName) {
								var _this3 = this;

								return _asyncToGenerator(function* () {
												return eos.contract.changeacc(username, newAccountName, eos.options).then(function () {
																return _this3.get(username);
												}).catch(function (err) {
																return false;
												});
								})();
				}

				/***
     * Loads up RIDL tokens on the identity
     * @param username
     * @param amount
     * @returns {Promise<T | boolean>}
     */
				loadtokens(username, amount) {
								var _this4 = this;

								return _asyncToGenerator(function* () {
												amount = parseFloat(amount.toString().split(' ')[0]).toFixed(4);
												if (amount <= 0) throw new Error("Amount must be greater than 0");

												return eos.writer.transaction([_eos.tokenCode, _eos.ridlCode], function (contracts) {
																contracts[_eos.tokenCode].transfer(eos.account.name, _eos.ridlCode, `${amount} RIDL`, '', eos.options);
																contracts[_eos.ridlCode].loadtokens(username, `${amount} RIDL`, eos.options);
												}).then(function () {
																return _this4.get(username);
												}).catch(function (err) {
																return false;
												});
								})();
				}

				/***
     * Claims a reserved identity
     * @param username
     * @param key
     * @param signature
     * @returns {Promise<T | void>}
     */
				claim(username, key, signature) {
								var _this5 = this;

								return _asyncToGenerator(function* () {
												return eos.contract.claim(eos.account.name, username, key, signature, eos.options).then(function () {
																return _this5.get(username);
												}).catch(function (err) {
																return console.error(err);
												});
								})();
				}

				identityBalance(username) {
								return _asyncToGenerator(function* () {
												return getIdentity(username).then(function (res) {
																return res.tokens;
												}).catch(function () {
																return null;
												});
								})();
				}

				accountBalance(name, asFloat = false) {
								return _asyncToGenerator(function* () {
												return yield eos.read({
																token: true,
																table: 'accounts',
																scope: name,
																limit: 1,
																firstOnly: true
												}).then(function (x) {
																return asFloat ? parseFloat(x.balance.split(' ')[0]) : x.balance;
												});
								})();
				}

				exists(name) {
								var _this6 = this;

								return _asyncToGenerator(function* () {
												return yield _this6.get(name).then(function (x) {
																return !!x;
												});
								})();
				}
}
exports.default = IdentityService;